import numpy as np
import itertools as it

def apply_2d_trafo(trafo_2d, array_5d, **kwargs):
    n_t, n_z, n_c, n_x, n_y = np.shape(array_5d)
    array_out_5d = None
    firstIteration = True

    for t, z, c in it.product(range(n_t), range(n_z), range(n_c)):
        result = trafo_2d(array_5d[t, z, c, :, :], **kwargs)
        if firstIteration:
            array_out_5d = np.zeros_like(array_5d, result.dtype)
            firstIteration = False
        array_out_5d[t, z, c, :, :] = result

    return array_out_5d


def apply_3d_trafo_zstack(trafo_3d, array_5d, **kwargs):
    n_t, n_z, n_c, n_x, n_y = np.shape(array_5d)
    array_out_5d = None
    firstIteration = True

    for t, c in it.product(range(n_t), range(n_c)):
        result = trafo_3d(array_5d[t, :, c, :, :], **kwargs)
        if firstIteration:
            array_out_5d = np.zeros_like(array_5d, **kwargs)
        firstIteration = False
        array_out_5d[t, :, c, :, :] = result

    return array_out_5d


def apply_3d_trafo_rgb(trafo_3d, array_5d, **kwargs):
    n_t, n_z, n_c, n_x, n_y = array_5d.shape
    array_out_5d = np.zeros_like(array5d, **kwargs)
    firstIteration = True

    for t, z in it.product(range(n_t), range(n_z)):
        result = trafo_3d(array_5d[t, z, :, :, :], **kwargs)
        if firstIteration:
            array_out_5d = array_out_5d.astype(result.dtype)
        firstIteration = False
        array_out_5d[t, z, :, :, :] = result

    return array_out_5d



import tifffile
import numpy as np
from xmltodict import parse
from apeer_ometiff_library import omexmlClass


def read_ometiff(input_path):
    with tifffile.TiffFile(input_path) as tif:
        array = tif.asarray()
        omexml_string = tif.ome_metadata

    # Turn Ome XML String to an Bioformats object for parsing
    metadata = omexmlClass.OMEXML(omexml_string)

    # Parse pixel sizes
    pixels = metadata.image(0).Pixels
    size_c = pixels.SizeC
    size_t = pixels.SizeT
    size_z = pixels.SizeZ
    size_x = pixels.SizeX
    size_y = pixels.SizeY

    # Expand image array to 5D of order (T, Z, C, X, Y)
    if size_c == 1:
        array = np.expand_dims(array, axis=-3)
    if size_z == 1:
        array = np.expand_dims(array, axis=-4)
    if size_t == 1:
        array = np.expand_dims(array, axis=-5)

    return array, omexml_string

def update_xml(omexml, Image_ID=None, Image_Name=None, Image_AcquisitionDate=None,
               DimensionOrder=None, dType=None, SizeT=None, SizeZ=None, SizeC=None, SizeX=None, SizeY=None,
                Channel_ID=None, Channel_Name=None, Channel_SamplesPerPixel=None):

    metadata = omexmlClass.OMEXML(omexml)

    if Image_ID:
        metadata.image().set_ID(Image_ID)
    if Image_Name:
        metadata.image().set_Name(Image_Name)
    if Image_AcquisitionDate:
        metadata.image().Image.AcquisitionDate = Image_AcquisitionDate

    if DimensionOrder:
        metadata.image().Pixels.DimensionOrder = DimensionOrder
    if dType:
        metadata.image().Pixels.PixelType = dType
    if SizeT:
        metadata.image().Pixels.set_SizeT(SizeT)
    if SizeZ:
        metadata.image().Pixels.set_SizeZ(SizeZ)
    if SizeC:
        metadata.image().Pixels.set_SizeC(SizeC)
    if SizeX:
        metadata.image().Pixels.set_SizeX(SizeX)
    if SizeY:
        metadata.image().Pixels.set_SizeY(SizeY)

    if Channel_ID:
        metadata.image().Channel.ID = Channel_ID
    if Channel_Name:
        metadata.image().Channel.Name = Channel_Name
    if Channel_SamplesPerPixel:
        metadata.image().Channel.SamplesPerPixel = Channel_SamplesPerPixel
    
    metadata = metadata.to_xml().encode()
    
    return metadata


def gen_xml(array):
    
    #Dimension order is assumed to be TZCYX
    dim_order = "TZCYX"
    
    metadata = omexmlClass.OMEXML()
    shape = array.shape
    assert ( len(shape) == 5), "Expected array of 5 dimensions"
    
    metadata.image().set_Name("IMAGE")
    metadata.image().set_ID("0")
    
    pixels = metadata.image().Pixels
    pixels.ome_uuid = metadata.uuidStr
    pixels.set_ID("0")
    
    pixels.channel_count = shape[2]
    
    pixels.set_SizeT(shape[0])
    pixels.set_SizeZ(shape[1])
    pixels.set_SizeC(shape[2])
    pixels.set_SizeY(shape[3])
    pixels.set_SizeX(shape[4])
    
    pixels.set_DimensionOrder(dim_order[::-1])
    
    pixels.set_PixelType(omexmlClass.get_pixel_type(array.dtype))
    
    for i in range(pixels.SizeC):
        pixels.Channel(i).set_ID("Channel:0:" + str(i))
        pixels.Channel(i).set_Name("C:" + str(i))
    
    for i in range(pixels.SizeC):
        pixels.Channel(i).set_SamplesPerPixel(1)
        
    pixels.populate_TiffData()
    
    return metadata.to_xml().encode()
    


def write_ometiff(output_path, array, omexml_string = None):
    
    if omexml_string is None:
        omexml_string = gen_xml(array)
        
    tifffile.imwrite(output_path, array,  photometric = "minisblack", description=omexml_string, metadata = None)
    



import unittest
import mock
import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import itertools as it
import numpy as np
import skimage
from apeer_ometiff_library import io

class TestsCore(unittest.TestCase):

    def test_funcName_condition_expectedResult(self):
        pass

if __name__ == '__main__':
    unittest.main()